function J=tv(I,iter,dt,ep,lam,I0)
%% Total Variation denoising.
%% Example: J=tv(I,iter,dt,ep,lam,I0)
%% Input: I    - image (double array gray level 1-256),
%%        iter - num of iterations,
%%        dt   - time step [0.2],
%%        ep   - epsilon (of gradient regularization) [1],
%%        lam  - fidelity term lambda [0],
%%        I0   - input (noisy) image [I0=I]
%%       (default values are in [])
%% Output: evolved image

[ny,nx]=size(I); ep2=0.05;%ep^2;

for i=1:iter,  %% do iterations
    % estimate derivatives
    I_x = I(:,[2:nx nx]) - I;
	I_y = I([2:ny ny],:) - I;
    I_xI = I - I(:,[1 1:nx-1]);
	I_yI = I - I([1 1:ny-1],:);
	%I_x = (I(:,[2:nx nx])-I(:,[1 1:nx-1]))/2;
	%I_y = (I([2:ny ny],:)-I([1 1:ny-1],:))/2;
	
    minmod_x = ((sign(I_x)+sign(I_xI))/2).*min(I_x,I_xI);
    minmod_y = ((sign(I_y)+sign(I_yI))/2).*min(I_y,I_yI);
    
    U = I_x ./ sqrt( ep2 + I_x.^2 + minmod_y.^2 );
    V = I_y ./ sqrt( ep2 + minmod_x.^2 + I_y.^2 );
    
    %U_x = U(:,[2:nx nx]) - U;
    %V_y = V([2:ny ny],:) - V;
    U_x = U - U(:,[1 1:nx-1]);
    V_y = V - V([1 1:ny-1],:);
    %U_x = (U(:,[2:nx nx]) - U(:,[1 1:nx-1]))/2;
    %V_y = (V([2:ny ny],:) - V([1 1:ny-1],:))/2;
   
   I_t = U_x + V_y + lam.*(I0-I);
   I=I+dt*I_t;  %% evolve image by dt
end % for i
%% return image
J=I;
