#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <benchmark.h>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <denoise/denoise_opencv.h>
#include <denoise/denoise_eigen.h>
#include <denoise/denoise_arma.h>

using namespace cv;
Mat denoise(Mat src);

int main( int argc, char** argv ) {
	
	// Benchmark parameters
	const int trials = 2;
	const int iterations = 5;

	char* imageName = argv[1];
	char* function = argv[2];

	Mat image, image_u;
	image_u = imread(imageName, 1);

	// Conversion to float
	image_u.convertTo(image, CV_32FC3);
	std::vector<Mat> channel(3);
	split(image, channel);	

	if( argc != 3 || !image.data ) {
		printf( " No image data \n " );
		return -1;
	}

	// Call to library
	Mat ch_denoised(image.rows,image.cols,CV_32F);
	std::vector<Mat> channel_denoised(3);

	double time = 0;
	if( *function == '1' ){
		time = benchmark(trials, iterations, [&]() {

			for(uint c = 0; c < channel.size(); c++) {
		
				ch_denoised = denoise_opencv(channel[c], 2.5f);
				channel_denoised[c] = ch_denoised;

			}

		});
	} else if ( *function == '2' ) {
		time = benchmark(trials, iterations, [&]() {

			for(uint c = 0; c < channel.size(); c++) {
		
				ch_denoised = denoise_eigen(channel[c], 2.5f);
				channel_denoised[c] = ch_denoised;

			}

		});
	} else if ( *function == '3' ) {
		time = benchmark(trials, iterations, [&]() {

			for(uint c = 0; c < channel.size(); c++) {
		
				ch_denoised = denoise_arma(channel[c], 2.5f);
				channel_denoised[c] = ch_denoised;

			}

		});
	} else {
		printf( "Formato de llamada no valido.\nEj: ./test_denoise [PATH_TO_IMAGE] [N]\n" );
	}

	Mat denoised_f;
	merge(channel_denoised, denoised_f);

	// Conversion to uchar
	Mat denoised;
	denoised_f.convertTo(denoised, CV_8UC3);

	// Show and save results
	printf("Time: %f \n", time);

	imwrite( "denoised.png", denoised );
	imwrite( "noised.png", image_u );

	namedWindow( imageName, CV_WINDOW_AUTOSIZE );
	namedWindow( "Denoised", CV_WINDOW_AUTOSIZE );

	imshow( imageName, image_u );
	imshow( "Denoised", denoised );

	waitKey(0);

	return 0;
}
