#include "Halide.h"

int main(int argc, char **argv) {

	Halide::ImageParam input_n0(Halide::Float(32), 3, "input_n0");
	Halide::ImageParam input0(Halide::Float(32), 3, "input0");

	// Variables
	Halide::Var x("x"), y("y"), c("c"), x_outer("x_outer"), y_outer("y_outer"), x_inner("x_inner"), y_inner("y_inner"), x_inner_outer("x_inner_outer"), y_inner_outer("y_inner_outer"), x_vectors("x_vectors"), y_pairs("y_pairs"), tile_index("tile_index"), subtile_index;
	
	// Funciones
	Halide::Func F1("F1"), Z("Z");
	Halide::Expr derx, dery, expr1, expr2, expr3;

	// Cmportamiento en los bordes
	Halide::Func input_n = Halide::BoundaryConditions::repeat_edge(input_n0);

	// ALGORITMO -----------------------
	derx = input_n(x, y, c) - input_n(x-1, y, c);
	dery = input_n(x, y, c) - input_n(x, y-1, c);

	expr1 = Halide::fast_inverse_sqrt( Halide::fast_pow(derx, 2) + Halide::fast_pow(dery, 2) + 0.01f);

	F1(x, y, c) = { derx * expr1, dery * expr1 };

	expr2 = F1(x+1, y, c)[0] - F1(x, y, c)[0] + F1(x, y+1, c)[1] - F1(x, y, c)[1];

	expr3 = expr2 + 2.5f*(input0(x, y, c) - input_n(x, y, c));

	Z(x, y, c) = ( input_n(x, y, c) + 0.01f*expr3 );

	// ESQUEMA -----------------------
	// 1) Codigo insertado
	// Por defecto
	
	// 2) Evaluacion en la raiz
	/*F1.compute_root();*/

	// 3) Evaluacion en mosaico
	/*Z.tile(x, y, x_outer, y_outer, x_inner, y_inner, 32, 32);
	Z.fuse(x_outer, y_outer, tile_index);
	Z.parallel(tile_index);
	Z.vectorize(x_inner, 8);
	F1.store_at(Z, tile_index).compute_at(Z, y_inner).vectorize(x, 8);*/

	// 4) Evaluacion por filas
	/*Z.split(y, y_outer, y_inner, 16);
	Z.parallel(y_outer);
	Z.vectorize(x, 8);
	F1.store_at(Z, y_outer).compute_at(Z, y_inner).vectorize(x, 8);*/
	
	Z.compile_to_file("denoising", {input_n0, input0});

	return 0;
}
